--
-- Tests -- examples from Tutorial - Chapter 1
--

-- Create Table (1.4.2.1)
CREATE TABLE SUPPLIER
  (SNO INTEGER,
   SNAME VARCHAR(20),
   CITY VARCHAR(20));

CREATE TABLE SELLS
   (SNO INTEGER,
   PNO INTEGER);

CREATE TABLE PART
  (PNO INTEGER,
   PNAME VARCHAR(20),
   PRICE DECIMAL(4,2));


-- Insert Into (1.4.3.1)
INSERT INTO SUPPLIER (SNO, SNAME, CITY) VALUES (1, 'Smith', 'London');
INSERT INTO SUPPLIER VALUES (2, 'Jones', 'Paris');
INSERT INTO SUPPLIER VALUES (3, 'Adams', 'Vienna');
INSERT INTO SUPPLIER VALUES (4, 'Blake', 'Rome');

INSERT INTO SELLS VALUES (1, 1);
INSERT INTO SELLS VALUES (1, 2);
INSERT INTO SELLS VALUES (2, 4);
INSERT INTO SELLS VALUES (3, 1);
INSERT INTO SELLS VALUES (3, 3);
INSERT INTO SELLS VALUES (4, 2);
INSERT INTO SELLS VALUES (4, 3);
INSERT INTO SELLS VALUES (4, 4);

INSERT INTO PART VALUES (1, 'Screw', 10);
INSERT INTO PART VALUES (2, 'Nut', 8);
INSERT INTO PART VALUES (3, 'Bolt', 15);
INSERT INTO PART VALUES (4, 'Cam', 25);

-- Simple selects (1.4.1.1)
SELECT * FROM PART WHERE PRICE > 10;
SELECT PNAME, PRICE FROM PART WHERE PRICE > 10;
SELECT PNAME, PRICE FROM PART WHERE PNAME = 'Bolt' AND
   ( PRICE = 0 OR PRICE <= 15);
SELECT PNAME, PRICE * 2 AS DOUBLE FROM PART WHERE PRICE * 2 < 50;


-- Joins (1.4.1.2)
SELECT S.SNAME, P.PNAME FROM SUPPLIER S, PART P, SELLS SE
  WHERE S.SNO=SE.SNO AND P.PNO = SE.PNO;

SELECT SNAME, PNAME FROM SUPPLIER 
   JOIN SELLS USING (SNO) JOIN PART USING (PNO); 

SELECT * FROM SUPPLIER JOIN SELLS USING (SNO) JOIN PART USING (PNO); 

-- Aggregate operators (1.4.1.3)
SELECT AVG(PRICE) AS AVG_PRICE FROM PART;

SELECT COUNT(PNO) FROM PART;

-- Aggregate by groups (1.4.1.4)
SELECT S.SNO, S.SNAME, COUNT(SE.PNO) FROM SUPPLIER S, SELLS SE
   WHERE S.SNO=SE.SNO GROUP BY S.SNO, S.SNAME;

-- Having (1.4.1.5)
SELECT S.SNO, S.SNAME, COUNT(SE.PNO) FROM SUPPLIER S, SELLS SE
   WHERE S.SNO=SE.SNO GROUP BY S.SNO, S.SNAME
   HAVING COUNT(SE.PNO) > 1;

-- Subqueries (1.4.1.6)
SELECT * FROM PART
   WHERE PRICE > (SELECT PRICE FROM PART
                  WHERE PNAME='Screw');

SELECT * FROM SUPPLIER S
   WHERE NOT EXISTS (SELECT * FROM SELLS SE
                     WHERE SE.SNO = S.SNO);

-- Subqueries in FROM (1.4.1.7)
SELECT MAX(subtable.avgprice)
   FROM (SELECT AVG(P.PRICE) AS avgprice
         FROM SUPPLIER S, PART P, SELLS SE
         WHERE S.SNO = SE.SNO AND
               P.PNO = SE.SNO
         GROUP BY S.SNO) subtable;

-- Union, Intersect, Except (1.4.1.8)
SELECT S.SNO, S.SNAME, S.CITY
   FROM SUPPLIER S
   WHERE S.SNAME = 'Jones'
UNION
   SELECT S.SNO, S.SNAME, S.CITY
   FROM SUPPLIER S
   WHERE S.SNAME = 'Adams';

SELECT S.SNO, S.SNAME, S.CITY
   FROM SUPPLIER S
   WHERE S.SNO > 1
INTERSECT
   SELECT S.SNO, S.SNAME, S.CITY
   FROM SUPPLIER S
   WHERE S.SNO < 3;

SELECT S.SNO, S.SNAME, S.CITY
   FROM SUPPLIER S
   WHERE S.SNO > 1
EXCEPT
   SELECT S.SNO, S.SNAME, S.CITY
   FROM SUPPLIER S
   WHERE S.SNO > 3;

-- Create Index (1.4.2.3)
CREATE INDEX I ON SUPPLIER (SNAME);

-- Create View (1.4.2.4)
CREATE VIEW London_Suppliers
   AS SELECT S.SNAME, P.PNAME
       FROM SUPPLIER S, PART P, SELLS SE
       WHERE S.SNO = SE.SNO AND
             P.PNO = SE.PNO AND
             S.CITY = 'London';

COMMENT ON VIEW London_Suppliers IS 'This is a list of suppliers from London';

SELECT * FROM London_Suppliers
   WHERE PNAME = 'Screw';

SELECT * FROM London_Suppliers;

-- Update (1.4.3.2)
UPDATE PART
   SET PRICE = 15
   WHERE PNAME = 'Screw';

SELECT * FROM PART;

-- Delete (1.4.3.3)
DELETE FROM SUPPLIER WHERE SNAME = 'Smith';

SELECT * FROM SUPPLIER;

-- Drop Table, Drop Index, Drop View (1.4.2.5)
DROP INDEX I;
DROP VIEW London_Suppliers;
DROP TABLE SUPPLIER;
DROP TABLE SELLS;
DROP TABLE PART;
